/***********************************************************************************************************************
* Copyright (C) . All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : main.c
* Version      :  
* Device(s)    : BAT32G135
* Tool-Chain   : MDK(armcc)
* Description  : This file is a template.
* Creation Date: 2019/4/30
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro Definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include <stdio.h>
#include <string.h>
#include "BAT32G135.h"
#include "userdefine.h"
#include "adc.h"
#include "sci.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint8_t   g_AdcIntTaken;      	/* adc interrupt flag */

double voltage = 3.0;
uint32_t g_ticks;
uint16_t get_value[8];
uint16_t avg_value;

int main()
{
	MD_STATUS status;
	uint32_t i;
	
//-----------------------------------------------------------------------
// Init UART0 for retarget printf/scanf etc. 
//----------------------------------------------------------------------- 
#if 1
	SystemCoreClockUpdate();
	status = UART0_Init(SystemCoreClock, 19200);
	if(status == MD_ERROR)
	{
		while(1);
	}
	
	printf("Hello, UART\n");
#endif

	
//-----------------------------------------------------------------------
// ADC Samples the voltage of Temperature Sensor
//-----------------------------------------------------------------------
	ADC_Init();					/* ADC inital according to macro defines in Macro Definitions section */
	
	avg_value = ADC_Converse(ADC_TEMPERSENSOR0, sizeof(get_value)/sizeof(get_value[0]), get_value);	
#if 1
	printf("ADC Value of Temperature Sensor:\n");
	for(i=0; i<sizeof(get_value)/sizeof(get_value[0]); i++)
	{
	    printf("Reference Voltage = %3.2fV, ADC Get Value = %04d, Voltage = %3.2fV\n", voltage, get_value[i], (double)get_value[i] * voltage / 4096);
	}
#endif

//-----------------------------------------------------------------------
// ADC Samples the voltage of Internal reference voltage (1.45V)
//-----------------------------------------------------------------------
	avg_value = ADC_Converse(ADC_INTERREFVOLT, sizeof(get_value)/sizeof(get_value[0]), get_value);	
#if 1
	printf("ADC Value of Internal reference voltage (1.45V):\n");
	for(i=0; i<sizeof(get_value)/sizeof(get_value[0]); i++)
	{
	    printf("Reference Voltage = %3.2fV, ADC Get Value = %04d, Voltage = %3.2fV\n", voltage, get_value[i], (double)get_value[i] * voltage / 4096);
	}	
#endif

//-----------------------------------------------------------------------
// ADC Samples the voltage of REGC (1.50V)
// Please connect REGC to P22 for SOP28 chip
//-----------------------------------------------------------------------
	avg_value = ADC_Converse(ADC_CHANNEL_2, sizeof(get_value)/sizeof(get_value[0]), get_value);	
#if 0
	printf("ADC Value of REGC:\n");
	for(i=0; i<sizeof(get_value)/sizeof(get_value[0]); i++)
	{
	    printf("Reference Voltage = %3.2fV, ADC Get Value = %04d, Voltage = %3.2fV\n", voltage, get_value[i], (double)get_value[i] * voltage / 4096);
#endif	
	ADC_Stop();	
}
/***********************************************************************************************************************

* Function Name: SysTick Handler
* Description  : Decreament the g_ticks value
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void SysTick_Handler(void)
{
	g_ticks--;
}

/***********************************************************************************************************************
* Function Name: HardFault_Handler
* Description  : Hard Fault handler to report stacked register values
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
// Hard Fault handler in C, with stack frame location and LR value
// extracted from the assembly wrapper as input parameters
void hard_fault_handler_c(unsigned int * hardfault_args, unsigned lr_value)
{
	unsigned int stacked_r0;
	unsigned int stacked_r1;
	unsigned int stacked_r2;
	unsigned int stacked_r3;
	unsigned int stacked_r12;
	unsigned int stacked_lr;
	unsigned int stacked_pc;
	unsigned int stacked_psr;
	
	stacked_r0 = ((unsigned long) hardfault_args[0]);
	stacked_r1 = ((unsigned long) hardfault_args[1]);
	stacked_r2 = ((unsigned long) hardfault_args[2]);
	stacked_r3 = ((unsigned long) hardfault_args[3]);
	stacked_r12 = ((unsigned long) hardfault_args[4]);
	stacked_lr = ((unsigned long) hardfault_args[5]);
	stacked_pc = ((unsigned long) hardfault_args[6]);
	stacked_psr = ((unsigned long) hardfault_args[7]);
	
	printf ("[Hard fault handler]\r\n");
	printf ("R0 = %x\r\n", stacked_r0);
	printf ("R1 = %x\r\n", stacked_r1);
	printf ("R2 = %x\r\n", stacked_r2);
	printf ("R3 = %x\r\n", stacked_r3);
	printf ("R12 = %x\r\n", stacked_r12);
	printf ("Stacked LR = %x\r\n", stacked_lr);
	printf ("Stacked PC = %x\r\n", stacked_pc);
	printf ("Stacked PSR = %x\r\n", stacked_psr);
	printf ("Current LR = %x\r\n", lr_value);
	
	while(1); // endless loop
}

/***********************************************************************************************************************
* Function Name: HardFault_Handler
* Description  : Assembly wrapper using Embedded Assembler in Keil MDK
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
// Hard Fault handler wrapper in assembly
// It extracts the location of stack frame and passes it to handler
// in C as a pointer. We also extract the LR value as second
// parameter.
__asm void HardFault_Handler(void)
{
		MOVS 	r0, #4
		MOV 	r1, LR
		TST 	r0, r1
		BEQ 	stacking_used_MSP
		MRS 	R0, PSP ; first parameter - stacking was using PSP
		B 		get_LR_and_branch
stacking_used_MSP
		MRS 	R0, MSP ; first parameter - stacking was using MSP
get_LR_and_branch
		MOV 	R1, LR ; second parameter is LR current value
		LDR 	R2,=__cpp(hard_fault_handler_c)
		BX 		R2
}

